/* 
 
 #    Decomposing graphs into edges and triangles
 #    
 #    Daniel Kral  Bernard Lidicky  Taisa L. Martins  Yanitsa Pehova
 #
 
Compile using  
g++
Optionally with -fopenmp
 
 
Files and file formats:
 
 Problem:
    min/max c_1xG_1 + c_2xG_2 + c_3xG_3
 subject to
    c_4xG_4 + c_5xG_5 + c_6 \geq 0
    c_7xG_7 + c_8xG_8 + c_9 \geq 0
 
 Objective file: (F_edges2__objective.txt)
   c_1 G_1
   c_2 G_2
   c_3 G_3
 0
   c_6
   c_4 G_5
   c_5 G_5
 0
   c_9
   c_7 G_7
   c_8 G_8
 0
 Optional program description (that does not start with a number)

 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <istream>
#include <iterator>
#include <vector>
#include <vector>
#include <utility>
#include <assert.h>
#include <cstring>
#include <algorithm>
#include <cstdarg>
#include <cstdlib>
#include <cmath>
#include <limits> 
#include <unistd.h>
#include <iomanip>  

using namespace std;

#define G_COLORED_EDGES 2

/*
#define USE_COUNT_INSTEAD_OF_DENSITY_IN_OBJECTIVE // When computing the density for objective function, it uses count instead of density. 
                                 // Usefull if you are shooting for exact result and want the SDP to have integers in the input. Alternatively, same could be
                                 // achieved by properly changing the objective function but it gives chance for rounding errors
#define G_USE_PERMITTED_SUBFLAGS // If the number of forbidden subflags of some size is too big, you may enumerate permitted subflags of some size in addition
                                  // to forbidden subflags
#define WRITE_INTEGER_DENSITIES_FOR_LINEAR_CONSTRAINTS  // it will modify the linear constraints by multiplying them by some constant and hope the numbers
                                  // from linear constraints that appear in the sdp will be integers

 
// Tweaks - maybe not used

#define G_REMOVE_DUPLICATES_WHILE_LOADING_FLAGS     // Usefull when loading some 'dirty' input
#define G_CHECK_FOR_FORBIDDEN_WHILE_LOADING_FLAGS   //
#define G_EARLY_FORBIDDEN_TEST_IN_GENERATING  // when coloring edges, test for forbidden graphs after every subgraph 0...x is entirely colored
#define G_FLAG_PRODUCTS_SLOW_LOW_MEMORY       // really slow, dont do this

 */


#ifndef V
#define V      8  // Maximum number of vertices of flags. If you know you need smaller, use smaller - will be faster and less memory consuming
#endif

#define G_NOT_ALL_FLAGS_USED // Some extra warnings if there are unexpected flags. This should be enabled

#define G_USING_ZERO_AS_ANY_COLOR // If something has color 0, it is considered to have anycolor or being uncolored. Usefull when trying to find extensions of some graph

#define G_BE_BRAVE_AND_IGNORE_SAFETY_ASSERTS // Will be slower, but if not defined. But if you don't trust the program, comment it :-)

#define USE_REDUCED_TYPES // 4 4 1:123  and 4 4 1:234 are considered redundant and only one is used. This you want to have on. Not having it is stupid unless you have ordered vertices


#define MAX_FLAG_TYPES  300000 // Just a constant - raise if neded - means different theta

//#define WRITE_INTEGER_DENSITIES_FOR_LINEAR_CONSTRAINTS

#ifdef G_COLORED_EDGES
#define COLORS_EDGES (G_COLORED_EDGES+1)
#endif


bool g_use_simple_linear_constraints = true;
vector<int>  g_use_simple_linear_constraints_list;




// This is the main class that holds a flag. 
class flag
{
public:
    flag()
    {
		m_Theta = 0;
		m_vertices = 0;
    }
    
    virtual ~flag()
    {
        
    }
    
    
    
  
    
    void make_all_vertices_labeled()
    {
        m_Theta = m_vertices;
    }
    
    bool is_labeled(const int v) const
    {
        assert(v < m_vertices);
        return  v < m_Theta;
    }
    
    // returns the number of labeled vertices
    int labeled_vertices_cnt() const
    {
        return m_Theta;
    }    

    
    // First m_Theta vertices are considered labeled in that order
    void set_Theta(int Theta)
    {
        // if !G_ORDERED_VERTICES, then the labeled vertices are first. Not true for ordered.
        assert(Theta <= m_vertices);
        m_Theta = Theta;
    }
    
	
    int get_Theta()
    {
        return m_Theta;
    }
    
    void set_vertices_and_Theta(int vertices, int Theta)
    {
        set_vertices(vertices);
        set_Theta(Theta);
    }
	
	
    // set the nymber of vertices and clears the graph
    void set_vertices(int vertices)
    {
        assert(vertices <= V);
        m_vertices = vertices;
        

		
#ifdef G_COLORED_EDGES
        for (int u = 0; u < V; u++)
        {
	        for (int v = 0; v < V; v++)
	        {
	            m_color_edge[u][v] = 0;
	        }
	    }
        
        m_colored_edges[0] = e();
        for (int c = 1; c < COLORS_EDGES; c++)  m_colored_edges[c] = 0;
#endif

        
    }


#ifdef G_COLORED_EDGES
    void color_edge(int u, int v, int color)
    {
    	assert(u != v);
    	assert(u < m_vertices);
    	assert(color >= 0);
        assert(color < COLORS_EDGES);
        
    	int old_color = m_color_edge[u][v];
        
    	m_colored_edges[old_color]--;
    	
        m_color_edge[u][v] = m_color_edge[v][u] = color;
    	
    	m_colored_edges[color]++;

    }
#endif
    
// Sequence of functions for checking if two flags have the same type //
private:
    // TODO: This sequence could be rewritten by using early check and no need
    // for creating copies of F - could be more effective.
    bool have_same_type_colorblind_colored_3edges(const flag &h) const
    {
        return have_same_type_colorblind_colored_edges(h);
    }
    
    
    bool have_same_type_colorblind_colored_edges(const flag &h) const
    {
        return have_same_type_colorblind_oriented_edges(h);
    }
    
    bool have_same_type_colorblind_oriented_edges(const flag &h) const
    {
        return have_same_type_colorblind_vertices(h);
    }
    
    bool have_same_type_colorblind_vertices(const flag &h) const
    {
        return have_same_type_not_colorblind(h);
    }

    bool have_same_type_not_colorblind(const flag &f) const
    {
        //if (m_Theta != f.m_Theta) return false;
        //if (m_Theta == 0) return true;
        
        
        
#ifdef G_COLORED_EDGES
        assert(m_Theta < V);
        for (int u = 0; u < m_Theta; u ++)
            for (int v = u+1; v < m_Theta; v++)
            {
#ifdef G_USING_ZERO_AS_ANY_COLOR
                if (m_color_edge[u][v] == 0 || f.m_color_edge[u][v] == 0) continue;
#endif            
                if (m_color_edge[u][v] != f.m_color_edge[u][v]) return false;
            }
#endif
        
        
        
        
        
        
        return true;
    }
    

public:    
    // This is the function to call in general program
    bool have_same_type(const flag &f) const
    {
        if (m_Theta == 0 && f.m_Theta == 0) return true;
        if (labeled_vertices_cnt() != f.labeled_vertices_cnt()) return false;
        
        return have_same_type_colorblind_colored_3edges(f);
    }
    
    
    bool is_isomorphic_to_colorblind_colored_3edges(const flag &h) const
    {
        return is_isomorphic_to_colorblind_colored_edges(h);
    }
    
    bool is_isomorphic_to_colorblind_colored_edges(const flag &h) const
    {
        return is_isomorphic_to_colorblind_oriented_edges(h);
    }
    
    bool is_isomorphic_to_colorblind_oriented_edges(const flag &h) const
    {
        return is_isomorphic_to_colorblind_vertices(h);
    }
    
    bool is_isomorphic_to_colorblind_vertices(const flag &h) const
    {
        return is_isomorphic_to_not_colorblind(h);
    }
    
    bool is_isomorphic_to(const flag &h) const
    {
        // Just a quick kill
        if (m_vertices != h.m_vertices) 
        {
            return false;
        }
        if (labeled_vertices_cnt() != h.labeled_vertices_cnt())
        {
            return false;
        }
        
        //cerr << "Testing " << h.print() << " and " << print() << endl;
        
        return is_isomorphic_to_colorblind_colored_3edges(h);
    }
	
	
	// perm is mapping vertices of h to vertices of this. pv is a candidate for
	// mapping of v, all vertices before v are already mapped.
	bool is_map_up_to_v_correct(int v, int pv, const int *perm, const flag &h) const
	{
			
						
#ifdef G_COLORED_EDGES
      // check edges, 0 as joker
      for (int u = 0; u < v; u++)
         if (m_color_edge[perm[u]][pv] != h.m_color_edge[u][v] && m_color_edge[perm[u]][pv] != 0 && h.m_color_edge[u][v] != 0) return false;        
#endif


        
        

        
		return true;
	}

    bool is_mapping_an_equality(int *perm, const flag &h) const
    {
#ifdef G_COLORED_EDGES
        //	check coloring
        for (int u = 0; u < m_vertices; u++)
        {
            for (int x = u+1; x < m_vertices; x++)
            {
                if (m_color_edge[perm[u]][perm[x]] == 0 || h.m_color_edge[u][x]==0) continue;
                if (m_color_edge[perm[u]][perm[x]] != h.m_color_edge[u][x]) return false;
            }
        }
#endif
        
        return true;
    }
    
    // Real isomorphism testing
	bool make_perm_isomorphic(int v, int *perm, bool *used, const flag &h) const
	{
		// Test if the permutation is isomorphism
		if (v >= m_vertices)
		{
#ifndef G_BE_BRAVE_AND_IGNORE_SAFETY_ASSERTS
            assert(is_mapping_an_equality(perm,h) == true);
#endif
            
			return true;
		}

		for (int pv = m_Theta; pv < m_vertices; pv++)
		{
			// check if used
			if (used[pv]) continue;

			if (!is_map_up_to_v_correct(v, pv, perm, h)) continue;

            
			perm[v] = pv;
			used[pv]  = true;
			
			if (make_perm_isomorphic(v+1, perm, used, h)) return true;
			
			used[pv]  = false;
		}
		
		return false;
	}
	
	
	bool is_isomorphic_to_not_colorblind(const flag &h) const
	{
        //if (labeled_vertices_cnt() > 0)
        //    cerr << "Starting testing not colorblind " << print() <<" and "<< h.print() << endl;

        
        // quick check
        if (m_vertices != h.m_vertices) 
        {
        if (labeled_vertices_cnt() > 0)
            //cerr << "Wrong number of vertices " << endl;
            return false;
        }
        if (labeled_vertices_cnt() != h.labeled_vertices_cnt())
        {
        if (labeled_vertices_cnt() > 0)
            //cerr << "Wrong number of labeled vertices " << labeled_vertices_cnt() << " " << h.labeled_vertices_cnt() << endl;
            return false;
        }
		
#ifdef G_COLORED_EDGES
        for (int c = 0; c < COLORS_EDGES; c++)
        {
            //if (m_vertices==0)
            //    cerr << c << "  " << m_colored_edges[c] << " " << h.m_colored_edges[c] << endl;
	        if (m_colored_edges[c] >  h.m_colored_edges[c]+h.m_colored_edges[0]) return false;
            if (m_colored_edges[c]+h.m_colored_edges[0] <  h.m_colored_edges[c]) return false;
	    }
#endif

		// Check for same type
		if (!have_same_type_not_colorblind(h)) return false;
		
		// try check all permutations --- WASTING HERE!!!!!!!!
		int perm[V];
		bool used[V];

		// Theta is preserved by the mapping
        for (int i = 0; i < m_Theta; i++)
        {
            perm[i] = i;
            used[i] = true;
        }
        
        for (int i = m_Theta; i < V; i++)
        {
            perm[i] = 0;
            used[i] = false;
        }
		
        return make_perm_isomorphic(m_Theta, perm, used, h);
	}

    
    void create_minlex_signature_blind_3edges(const flag &h, string &lex_min) const
    {
        string Hprint = h.print("");
        if (Hprint.compare(lex_min) < 0)
        {
            lex_min = Hprint;
        }
    }
    
    void create_minlex_signature_blind_edges(const flag &h, string &lex_min) const
    {
        create_minlex_signature_blind_3edges(h, lex_min);
    }
    
    void create_minlex_signature_oriented_edges(const flag &h, string &lex_min) const
    {
        create_minlex_signature_blind_edges(h,lex_min);
    }
    
    
    void create_minlex_signature_blind_vertices(const flag &h, string &lex_min) const
    {
        create_minlex_signature_oriented_edges(h, lex_min);
    }

        
    void create_minlex_signature()
    {
    }


    void get_type_subflag(flag &f) const
    {
        const int mapping[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
        f.as_subflag(*this,mapping,m_Theta,m_Theta);        
    }
    
    
    // removes two vertices
    void remove_vertices(int u, int v)
    {
        int mapping[V];
        
        int id = 0;
        for (int i = 0; i < m_vertices;i++)
        {
            if (i == u || i == v) continue;
            mapping[id++] = i;
        }
        
        flag h;
        h.as_subflag(*this, mapping, m_vertices-2, 0);
        
        *this = h;
    }

    void remove_labeled_vertices()
    {
        if (m_Theta == 0)
            return;
        
        int mapping[V];
        
        for (int i = 0; i < m_vertices-m_Theta;i++)
        {
            mapping[i] = i+m_Theta;
        }
        
        flag h;
        h.as_subflag(*this, mapping, m_vertices-m_Theta, 0);
        
        *this = h;
    }
    
    void as_subflag(const flag &h,const int *mapping, int vertices, int theta, bool create_signature=true)
    {
        set_vertices_and_Theta(vertices,theta);
		
#ifdef G_COLORED_EDGES
        for (int x = 0; x < m_vertices-1; x++)
            for (int y = x+1; y < m_vertices; y++)
            {
                color_edge(x,y,h.m_color_edge[mapping[x]][mapping[y]]);
            }
#endif

    }
    


    
    
        
    int e()
    {
        return (m_vertices*(m_vertices-1))/2;
    }
    
    int e3()
    {
        return (m_vertices*(m_vertices-1)*(m_vertices-2))/6;
    }	
	
	
    bool contains_as_subflag_map_rest(const flag &g, int* mapping, int next_to_map, int first_available) const
    {
        if (next_to_map >= g.m_vertices)
        {
            flag h;
            h.as_subflag(*this, mapping,g.m_vertices, g.m_Theta);
            
            //This may happen!!
            // assert(g.m_Theta == 0);
         
            //cerr << "Testing " << h.print() << " and " << g.print() << endl;

            
            if (h.is_isomorphic_to(g))
            {
                return true;
            }
                         
            return false;
        }
        
        if (first_available < g.m_Theta)
        {
            mapping[next_to_map] = first_available;
            if (contains_as_subflag_map_rest(g,mapping,next_to_map+1,first_available+1)) return true;            
        }
        else
        {
        for (int i = first_available; i < m_vertices; i++)
        {
            mapping[next_to_map] = i;
            if (contains_as_subflag_map_rest(g,mapping,next_to_map+1,i+1)) return true;
        }
        }
        
        return false;
    }
    
    // Count density of g as a subflags
    bool contains_as_subflag(const flag &g) const
    {
        if (g.m_vertices > m_vertices) return false;
        
        int mapping[g.m_vertices];
                
        return contains_as_subflag_map_rest(g,mapping,0,0);        
    }
    
    
	void density_subflag_map_rest(const flag &g, int &total, int &good, int* mapping, int next_to_map, int first_available) const
	{
		if (next_to_map >= g.m_vertices)
		{
			flag h;
			h.as_subflag(*this, mapping,g.m_vertices, g.m_Theta);
			
			assert(g.m_Theta == 0);
			
			if (h.is_isomorphic_to(g)) good++;
			
			total++;
			return;
		}
		
		for (int i = first_available; i < m_vertices; i++)
		{
			mapping[next_to_map] = i;
			density_subflag_map_rest(g,total,good,mapping,next_to_map+1,i+1);
		}
	}
	
	// Count density of g as a subflags
    double density_subflag(const flag &g) const
    {
		if (g.m_vertices > m_vertices) return 0;
		
		int mapping[g.m_vertices];
		
		int total = 0;
		int good = 0;
		
		density_subflag_map_rest(g,total,good,mapping,0,0);
		
        return (double)good/(double)total;
	}

	int count_subflag(const flag &g) const
	{
		if (g.m_vertices > m_vertices) return 0;
		
		int mapping[g.m_vertices];
		
		int total = 0;
		int good = 0;
		
		density_subflag_map_rest(g,total,good,mapping,0,0);
		
      return good;
	}
    
    
    void count_labeled_copies_of_map_rest(const flag &g_labeled, int &good_maps, int* mapping, int *used, int next_to_map) const
    {
        if (next_to_map >= g_labeled.m_vertices)
        {
            flag h;
            h.as_subflag(*this, mapping,g_labeled.m_vertices, g_labeled.m_Theta);
            
            assert(g_labeled.m_Theta == g_labeled.m_vertices);
            
            if (h.have_same_type(g_labeled)) good_maps++;

            return;
        }
        
        for (int i = 0; i < m_vertices; i++)
        {
            if (used[i]) continue;
            mapping[next_to_map] = i;
            used[i] = 1;
            count_labeled_copies_of_map_rest(g_labeled,good_maps,mapping,used,next_to_map+1);
            used[i] = 0;
        }
    }
    

    int count_labeled_copies_of(const flag &g) const
    {
        if (g.m_vertices > m_vertices) return 0;
       
        // This does not work correctly for unlabeled graphs
        assert(g.m_Theta == 0 && m_Theta == 0);
        
        
        // We use have_same_type to check correct mappings...
        flag g_labeled = g;
        g_labeled.m_Theta = g.m_vertices;
        
        int mapping[g.m_vertices];
        int used[m_vertices];
        for (int i = 0; i < m_vertices; i++)
        {
            used[i] = 0;
        }
        
        int good_maps = 0;
        
        count_labeled_copies_of_map_rest(g_labeled,good_maps,mapping,used,0);
        
        return good_maps;
    }
    
    
    
    void generate_subflags_of_size_n_rest(int n, vector<flag> &subflags, int* mapping, int next_to_map, int first_available, const vector<int> &available_to_map) const
    {
        if (next_to_map >= n)
        {
            flag h;
            h.as_subflag(*this, mapping, n, 0);
            bool h_is_new = true;
            for (int i = 0; i < (int)subflags.size(); ++i)
            {
                if (h.is_isomorphic_to(subflags[i])) 
                {
                    h_is_new = false;
                    break;
                } 
            }
            if (h_is_new) subflags.push_back(h);            
            return;
        }
        
        for (int i = first_available; i < (int)available_to_map.size()-(n-next_to_map); i++)
        {
            mapping[next_to_map] = available_to_map[i];
            generate_subflags_of_size_n_rest(n,subflags,mapping,next_to_map+1,i+1,available_to_map);
        }
    }
   
    
    // Could be written as a template with size of in_subflag - should be faster
    // an/or with arrays
    // if in_subrgaph is specified, then only subflags containing in_subrgaph are generated
    int generate_subflags_of_size_n(int n, vector<flag> &subflags, const vector<int> &in_subflag = vector<int>()) const
    {
        if (n > m_vertices) return 0;
        
        // This does not work correctly for unlabeled graphs
        assert(m_Theta == 0);
                
        int mapping[n];
        
        vector<int> available_to_map;
        for (int i = 0; i < m_vertices; i++)
            if (find(in_subflag.begin(), in_subflag.end(), i) == in_subflag.end()) 
                available_to_map.push_back(i);
        
        for (int i = 0; i < (int)in_subflag.size(); i++)
            mapping[i] = in_subflag[i];
        
        generate_subflags_of_size_n_rest(n, subflags, mapping, (int)in_subflag.size(),0,in_subflag);
        
        return (int)subflags.size();
    }
    
    
    
    
    // Copy data from g if the number of vertices of g is <= current number
    void copy_from(const flag &g)
    {
        assert(m_vertices >= g.m_vertices);
     
        

        
        
#ifdef G_COLORED_EDGES
        for (int x = 0; x < g.m_vertices-1; x++)
            for (int y = x+1; y < g.m_vertices; y++)
            {
                color_edge(x,y,g.m_color_edge[x][y]);
            }        
#endif        
    }
    
/***************************************************************************************************************** Printing ****************/
	void print_for_human() const
	{
		// Prints the number of vertices, zero as not type and then part of the diagonal matrix
		cout << m_vertices << " " << m_Theta << " ";
		
		
		cout << endl;
	}

    template<typename T>
    string print_latex(bool use_label, const T &graph_label) const
    {
        if (m_vertices == 0)
        {
            return "";
        }
        
        stringstream ss;
        
        // Prints the number of vertices, zero as not type and then part of the diagonal matrix
        ss << "\\begin{tikzpicture}\\outercycle{" << m_vertices+1 << "}{" << m_vertices << "}\n";
        
#ifdef G_COLORED_EDGES
        for (int u = 0; u < m_vertices; u++)
        {
            for (int v = u+1; v < m_vertices; v++)
            {
                ss << "\\draw[edge_color" << m_color_edge[u][v] << "] (x"<<u<<")--(x"<<v<<");"; 
            }
            ss << "  ";
        }
        ss << endl;
#endif
      
        for (int i = 0; i < m_vertices; i++)
        {
            if (i < m_Theta)
                ss << "\\draw (x"<<i<<") node[labeled_vertex]{};";
            else
                ss << "\\draw (x"<<i<<") node[unlabeled_vertex]{};";
        }
        ss << endl;
        
        
        if (use_label)
        {
            ss << "\\draw (labelpoint) node{"<<graph_label<<"};" << endl;
        }
        ss << "\\end{tikzpicture}" << endl;
        
        return ss.str();
    }
    
    
	
	string print(const string &delimeter=" ") const
	{
		stringstream ss;
		
		// Prints the number of vertices, zero as not type and then part of the diagonal matrix
		ss << m_vertices << delimeter << m_Theta << delimeter << delimeter;

#ifdef G_COLORED_EDGES
  		for (int u = 0; u < m_vertices; u++)
  		{
    		for (int v = u+1; v < m_vertices; v++)
				ss << delimeter << m_color_edge[u][v];
            ss << delimeter;
		}
#endif
        
		return ss.str();
	}
    
	
	
    
/***************************************************************************************************************** Loading ****************/	
	void load_from_string(const char *str)
	{
		stringstream s(str);
		load_from_stream(s, -1, -1);
	}
		
	
   bool load_from_stream(istream &stream, int assumed_vertices, int assumed_theta)
   {
		int vertices, theta;
		stream >> vertices >> theta;
		if (!stream.good()) return false; 

#ifdef G_VERBOSE_OUTPUT
        if (assumed_theta != -1 && theta != assumed_theta)
        {
            cerr << "Err: assumed theta " << assumed_theta << " but loaded " <<  theta << " for " << vertices << " vertices " << endl;
        }
#endif       
		assert(assumed_theta == -1 || theta == assumed_theta);
        assert(assumed_vertices == -1 || assumed_vertices == vertices);
		set_vertices_and_Theta(vertices, theta);
		
		
#ifdef G_COLORED_EDGES
		int color;
		for (int u = 0; u < vertices; u++)
		{
			for (int v = u+1; v < vertices; v++)
			{
				stream >> color;
				color_edge(u,v,color);
			}
		}
#endif
       
       //cerr << print() << endl;
       
       create_minlex_signature();

		return true;
	}

public:
    
    flag& operator=(const flag& f)
    {
        m_vertices = f.m_vertices;
        m_Theta    = f.m_Theta;

        
        
        
#ifdef G_COLORED_EDGES
        for (int i = 0; i < m_vertices; i++)
            for (int j = 0; j < m_vertices; j++)
                {
                    m_color_edge[i][j] = f.m_color_edge[i][j];
                }
        
        for (int i = 0; i < COLORS_EDGES; i++)
        {
            m_colored_edges[i] = f.m_colored_edges[i];
        }         
        
#endif
        
        return *this;
    }
    
		
	int m_vertices;  // number of vertices
    int m_Theta;     // number of labeled vertices in case of not ordered. Binary string which are labeled in ordered
 	
//	int m_id; // id of the original flag

    
	
	
#ifdef G_COLORED_EDGES
	int m_color_edge[V][V];  // adjacency matrix
//	int m_colored_deg[V][COLORS_EDGES]; // number of edges from each vertex of a particular color
	int m_colored_edges[COLORS_EDGES]; // number of edges of each color
//	vector<int> m_deg_sorted;
//	bool m_deg_sorted_valid;
#endif
 
    
};


class flag_and_coefficient
{
public:
	flag   g;
	double coefficient;
};


// Try if the number should be actuallly rounded
double smart_round(double d)
{
    double dr = round(d);
    if (abs(dr-d) < 0.00000001) return dr;
    return d;
}


// constraint is of the form...  m_flag + m_constant >= 0
class linear_constraint
{
public:
    linear_constraint()
    {
        m_checked = false; 
        m_labeled_vertices_in_type_cnt = 0;
    }
    
    
    bool check_constraint()
    {
        if (m_entries.size() == 0) 
        {
            m_checked = false;
            //m_same_types = false;
            return false;
        }
        
        m_entries_max_size = m_entries[0].g.m_vertices;
        
        for (int i = 1; i < (int)m_entries.size(); i++)
        {
            if (! m_entries[0].g.have_same_type(m_entries[i].g))
            {
                m_checked = false;
                return false;
            }
            if (m_entries_max_size < m_entries[i].g.m_vertices)
            {
                m_entries_max_size = m_entries[i].g.m_vertices;
            }
        }
        
        m_labeled_vertices_in_type_cnt = m_entries[0].g.labeled_vertices_cnt();
        
        m_entries[0].g.get_type_subflag(m_type);
        
        m_checked = true;

//        m_same_types = true;
        
        if (m_constant != 0)
        {
            flag_and_coefficient fc;
            fc.g = m_type;
            fc.coefficient = m_constant;
            m_entries.push_back(fc);
            m_constant = 0;
        }
        
        return false;
    }
    
    string print()
    {
        stringstream ss;
        ss << "0 " << m_constant << endl;
        for (int i = 0; i < (int)m_entries.size(); i++)
        {
            ss.precision(17);
            ss << m_entries[i].coefficient << " " << m_entries[i].g.print() << endl;
        }
        return ss.str();
    }
    
    void add_entry(const flag_and_coefficient& fc)
    {
        for (int i = 0; i < (int)m_entries.size(); i++)
        {
            if (m_entries[i].g.is_isomorphic_to(fc.g))
            {
                m_entries[i].coefficient += fc.coefficient;
                if (m_entries[i].coefficient == 0)
                {
                    m_entries.erase(m_entries.begin()+i);
                }
                return;
            }
        }
        m_entries.push_back(fc);
    }
    
public:
	vector<flag_and_coefficient> m_entries;
	double m_constant;
    
    bool   m_checked;
    //bool   m_same_types;  // true if the entries are of same type and size 
    int    m_entries_max_size;
    int    m_labeled_vertices_in_type_cnt;
    
    flag   m_type;
};




vector<flag> g_unlabeled_flags[V+1];
vector<flag> g_types[V+1]; // types - it is just same as g_unlabeled_flags, but all vertices are labeled :-) Used for reduced types
vector<vector<flag> >  g_flags; // flags to process using multiplication - every type&size is a separate list


vector<linear_constraint> g_linear_constraints;
vector<flag_and_coefficient> g_objective_combination;
vector<flag> g_forbidden_subflags;
vector<flag> g_forbidden_subflags_by_size[V+1];
vector<flag> g_no_slack_flags; // means no slack in CSDP
string       g_program_description;  // loaded from the objective function







int binomial(int n, int k) {
    int b = 1;
    for (int i = 0; i < k; ++i) {
        b *= (n - i);
        b /= (i + 1);
    }
    return b;
}

int factorial(int n) {
    int b = 1;
    for (int i = 2; i <= n; i++) {
        b *= i;
    }
    return b;
}


#ifdef DONT_USE_C11
double stod(const string&  str)
{
    stringstream ss(str);
    double d;
    ss >> d;
    return d;
}

string to_string (long long val)
{
    stringstream ss;
    ss << val;
    return ss.str();
}
#endif


string filename_prefix()
{
    stringstream filename;
    
    filename << "F"
#ifdef G_COLORED_EDGES
    << "_edges" << COLORS_EDGES-1
#endif
    ;
    return filename.str();
}




inline int find_flag_in_list(const flag &f, const vector<flag> &fv)
{
    for (int i = 0; i < (int)fv.size(); ++i)
    {
        if (f.is_isomorphic_to(fv[i]))
        {
            return i;
        }
    }
    return -1;
}

inline int get_flag_type_in_list(const flag& f, vector< vector<flag> > &flags)
{
#ifdef USE_REDUCED_TYPES
    const int identity[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30};
    flag f_type;
    f_type.as_subflag(f,identity,f.m_Theta,0);
#endif 
    
    // Find correct type
    for (int i = 0; i < (int)flags.size(); i++)
    {
        if (flags[i].size() == 0)
        {
            cerr << "Some bug" << endl;
        }
        if (f.have_same_type(flags[i][0])) return i;
#ifdef USE_REDUCED_TYPES
        if (f.m_Theta == flags[i][0].m_Theta)
        {
            flag g_flags_type;
            g_flags_type.as_subflag(flags[i][0],identity,flags[i][0].m_Theta,0);
            if (g_flags_type.is_isomorphic_to(f_type))
                return -2;
        }
#endif        
        
    }
    
    return -1;
}

inline void include_flag_in_list(const flag& f, vector< vector<flag> > &flags, int id = -3, bool ignore_reduced_types = true)
{
    if (id == -3)
    {
        id = get_flag_type_in_list(f, flags);
    }
#ifdef  USE_REDUCED_TYPES   
    if (id == -2 && ignore_reduced_types) return;
#endif   
    if (id == -1 || id == -2)
    {
        vector<flag> new_type_list;
        new_type_list.push_back(f);
        flags.push_back(new_type_list);
        return;
    }
    
    flags[id].push_back(f);
}


inline void include_flag_in_list_if_new(const flag& f, vector< vector<flag> > &flags, bool ignore_reduced_types = true)
{
    int id = get_flag_type_in_list(f, flags);
#ifdef USE_REDUCED_TYPES
    if (id == -2 && ignore_reduced_types) 
    {
        return;
    }
#endif
    if (id == -1 || id == -2)
    {
//        cerr << "new type" << endl;
        include_flag_in_list(f, flags, id,ignore_reduced_types);
        return;
    }
    
    if (find_flag_in_list(f,flags[id]) != -1)
    {
//        cerr << "duplicate" << endl;
        return;   
    }
//    cerr << "new for known type" << f.print() << endl;
    flags[id].push_back(f);
}





inline bool is_flag_forbidden(const flag &g)
{
    for (int i = 0; i < (int)g_forbidden_subflags.size(); i++)
    {
        //cerr << "Testing " << g.print() << " and " << g_forbidden_subflags[i].print() << endl;
        if (g.contains_as_subflag(g_forbidden_subflags[i])) 
        {
            //cerr << "Forbidding " << g.print() << " because of " << g_forbidden_subflags[i].print() << endl;
            return true;
        }
    }
    return false;
}

//#ifdef G_USE_PERMITTED_SUBFLAGS  not used in early generation - maybe should be... ?
inline bool is_flag_forbidden_avoid_zeros(const flag &g, const vector<int> &in_subrgaph = vector<int>())
{
    vector<flag> subflags_list;
    for (int i = 1; i <= g.m_vertices; i++)
    {
        //if (g_permitted_subflags[i].size() == 0) continue;        
        if (g_forbidden_subflags_by_size[i].size() == 0) continue;
        
        subflags_list.clear();
        g.generate_subflags_of_size_n(i, subflags_list, in_subrgaph);
                
        for (int j = 0; j < (int)subflags_list.size(); j++)
        {
// Do not check subgraphs that are not fully colored
#ifdef G_COLORED_EDGES            
            if (subflags_list[j].m_colored_edges[0] > 0) continue;
#endif            
            if (find_flag_in_list(subflags_list[j], g_forbidden_subflags_by_size[i]) != -1) return true;
        }
    }
    return false;
}

/*
inline bool is_flag_forbidden_noninduced(const flag &g)
{
    for (int i = 0; i < (int)g_forbidden_subflags.size(); i++)
    {
        if (g.has_as_notinduced_subflag(g_forbidden_subflags[i])) return true;
    }
    return false;
}
*/

inline bool g_already_in_known_flags(flag &g, vector<flag> &flag_list)
{
    for (unsigned int i = 0; i < flag_list.size(); i++)
    {
        if (flag_list[i].is_isomorphic_to(g))
        {
            return true;
        }
    }
    return false;
}

inline void add_g_to_known_flags(flag &g, vector<flag> &flag_list)
{
    flag_list.push_back(g);
    //if (g.m_3edges_cnt >= 20)
    {
    //    cerr << g.print() << endl;
    }
}


#ifdef G_COLORED_EDGES
void try_color_edge(flag &g, int u, int v, vector<flag> &flag_list)
{
    if (v >= g.m_vertices || v == u)
    {
        u++;
        v = u+1;
#ifdef G_EARLY_FORBIDDEN_TEST_IN_GENERATING
        // test if graph on vertices 0...u is not forbidden
        const int identity[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
        flag g_to_u;
        g_to_u.as_subflag(g,identity, u+1, 0);
        
        cerr << "Testing " << g_to_u.print() << endl;
        
        if (is_flag_forbidden(g_to_u)) return;
#endif
    }
    if (v >= g.m_vertices)
    {
        if (is_flag_forbidden(g)) return;
        
        g.create_minlex_signature();
        
        if (!g_already_in_known_flags(g,flag_list)) add_g_to_known_flags(g,flag_list);
        //		add_g_to_known_flags(flagh_list);
        return;
    }
    
    
    // Color only uncolored edges.....
    if (g.m_color_edge[u][v] == 0)
    {
// Either color 1 us used for blow-up or for tournaments it is not used at all
// since color 1 does not change orientation
#if defined(G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY) || defined(G_TOURNAMENT)         
        for (int color = 2; color < COLORS_EDGES; color++)
#else
        for (int color = 1; color < COLORS_EDGES; color++)
#endif
        {
            g.color_edge(u,v,color);
            try_color_edge(g,u,v+1,flag_list);
        }
        g.color_edge(u,v,0);
    }
    else
    {
        try_color_edge(g,u,v+1,flag_list);
    }
}
#endif







void extensions_of_g_edges(flag &g, vector<flag> &flag_list)
{
    //cerr << "Generating extensions of " << g.print() << endl;
    
#ifdef G_COLORED_EDGES
    try_color_edge(g,0,1,flag_list);
    return;
#endif    
}





void extensions_of_g(flag &g, vector<flag> &flag_list)
{
    extensions_of_g_edges(g,flag_list);
}



// u is original, v will be a copy
// edges containing both will have color 1
void duplicate_vertex_u_to_v(flag &g, int u, int v)
{

#ifdef G_COLORED_EDGES
    
    for (int x = 0; x < g.m_vertices-1; x++)
    {
        if (x == u) 
            g.color_edge(v,x,1);
        else
            g.color_edge(v,x,g.m_color_edge[u][x]);
    }
#endif


    
}





///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)


// Rest of F2 is also piskced as a subset
void pick_F2_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_F2, int next_to_map,  bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F2.m_vertices)
    {
        flag H_F2;
        H_F2.as_subflag(H, mapping_F2, F2.m_vertices, F2.m_Theta);
        if (!H_F2.is_isomorphic_to(F2)) return;
		
        good_maps++;
        
        return;
    }
    
    int min_vertex = 0;
    if (next_to_map > F2.m_Theta) min_vertex = mapping_F2[next_to_map-1]+1;
    
    for (int v = min_vertex; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        used_from_H[v] = true;
        mapping_F2[next_to_map] = v;
        pick_F2_mapping(F1,F2,H,mapping_F2,next_to_map+1,used_from_H,good_maps);
        used_from_H[v] = false;
    }
    
}

// Rest of F1 is picked only as a subset
void pick_F1_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_F1, int *mapping_F2, int next_to_map,  bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F1.m_vertices)
    {
		//cout << "." << endl;

        // Check if the mapping is corect!
        flag H_F1;
        H_F1.as_subflag(H, mapping_F1, F1.m_vertices,F1.m_Theta);
		
        if (!H_F1.is_isomorphic_to(F1)) return;
        
        pick_F2_mapping(F1,F2,H,mapping_F2,F2.m_Theta,used_from_H,good_maps);
        return;
    }
    
    int min_vertex = 0;
    if (next_to_map > F1.m_Theta) min_vertex = mapping_F1[next_to_map-1]+1;
    
    for (int v = min_vertex; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        used_from_H[v] = true;
        mapping_F1[next_to_map] = v;
        pick_F1_mapping(F1,F2,H,mapping_F1,mapping_F2,next_to_map+1,used_from_H,good_maps);
        used_from_H[v] = false;
    }
}

// Theta mapping is picked as all permutations...
void pick_theta_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_theta, int next_to_map, bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F1.m_Theta)
    {
        int mapping_F1[F1.m_vertices];
        int mapping_F2[F2.m_vertices];
        for (int i = 0; i < F1.m_Theta; i++)
        {
            mapping_F2[i] = mapping_F1[i] = mapping_theta[i];
        }
				
        pick_F1_mapping(F1,F2,H,mapping_F1,mapping_F2, F1.m_Theta, used_from_H,good_maps);
        return;
    }
	
    for (int v = 0; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        if (!H.is_map_up_to_v_correct(next_to_map, v, mapping_theta, F1)) continue;
       
         used_from_H[v] = true;
         mapping_theta[next_to_map] = v;
         pick_theta_mapping(F1,F2,H,mapping_theta,next_to_map+1,used_from_H,good_maps);
		 used_from_H[v] = false;
    }
}

inline double P_F1_F2_IN_H(const flag &F1, const flag &F2, const flag &H)
{
    assert(F1.m_Theta == F2.m_Theta);

#if defined(G_COLORED_EDGES) && !defined(G_COLORED_EDGES_BLIND)
	for (int c = 1; c < COLORS_EDGES; c++)
	{
        //		cout << "e" << endl;
        if (F1.m_colored_edges[c] > H.m_colored_edges[c]) return 0;
        if (F2.m_colored_edges[c] > H.m_colored_edges[c]) return 0;
	}
#endif

		
    int mapping_theta[F1.m_Theta];
    bool used_from_H[H.m_vertices];
    
    for (int i = 0; i < H.m_vertices; i++)
    {
        used_from_H[i] = false;
    }
	
    int good_maps = 0;
    pick_theta_mapping(F1,F2,H,mapping_theta,0,used_from_H,good_maps);
	
    
    int all_maps = 1;
#ifndef WRITE_INTEGER_DENSITIES_FOR_FLAGS
    for (int i = 0; i < F1.m_Theta; i++) all_maps *= (H.m_vertices-i);
    all_maps *= binomial(H.m_vertices-F1.m_Theta,F1.m_vertices-F1.m_Theta);
    all_maps *= binomial(H.m_vertices-F1.m_vertices,F2.m_vertices-F1.m_Theta);
#endif
    
	//    if (good_maps > 0)
	//        cout << "good_maps/all_maps: " << good_maps << "/" << all_maps << endl;
    
    
    return (double)good_maps/(double)all_maps;
}



inline double P_F1_F2_F3_IN_H(const flag &F1, const flag &F2, const flag &F3, const flag &H)
{
    assert(F1.m_Theta == F2.m_Theta);
    assert(F1.m_Theta == F3.m_Theta);
    
    assert(F1.m_vertices+F2.m_vertices+F3.m_vertices - 2*F1.labeled_vertices_cnt() <= H.m_vertices);
    
    flag type;
    F1.get_type_subflag(type);    

    
    // Some easy checks...
#if defined(G_COLORED_EDGES) && !defined(G_COLORED_EDGES_BLIND)
    for (int c = 1; c < COLORS_EDGES; c++)
    {
        if (F1.m_colored_edges[c] + F2.m_colored_edges[c] + F3.m_colored_edges[c] - 2*type.m_colored_edges[c] > H.m_colored_edges[c]) return 0;
    }
#endif
    

    int good_maps = 0;
    int all_maps = 0;
    
    int vertex_split[V];

    int t = 0;
    for (int i = 0; i < type.m_vertices; i++) vertex_split[t++] = i;
    for (int i = type.m_vertices; i < F1.m_vertices; i++) vertex_split[t++] = -1;
    for (int i = type.m_vertices; i < F2.m_vertices; i++) vertex_split[t++] = -2;
    for (int i = type.m_vertices; i < F3.m_vertices; i++) vertex_split[t++] = -3;
    for (;t < H.m_vertices; t++) vertex_split[t] = -4;
    
    std::sort (vertex_split,vertex_split+H.m_vertices);
    
    flag tmp;
    int mapping[V];
    int next_to_map; // temporary
    do {
        all_maps++;
        
        
        //cerr << "split: ";
        //for (int i = 0; i < H.m_vertices; i++)
        //    cerr << vertex_split[i] << " " ;
        //cerr << endl;        
        
        
        // check if Theta correct
        for (int j = 0; j < H.m_vertices; j++)
        {
            if (vertex_split[j] < 0) continue;
            if (vertex_split[j] < type.m_vertices) mapping[vertex_split[j]] = j;
        }
        tmp.as_subflag(H,mapping,type.m_vertices,type.m_Theta);
        //cerr << tmp.print() << " vs " << type.print();
        if (!tmp.is_isomorphic_to(type)) continue;

        
        // check if F1 correct
        next_to_map = type.m_vertices;
        for (int j = 0; j < H.m_vertices; j++)  if (vertex_split[j] == -1) mapping[next_to_map++] = j;
        tmp.as_subflag(H,mapping,F1.m_vertices,type.m_Theta);
        if (!tmp.is_isomorphic_to(F1)) continue;


        // check if F2 correct
        next_to_map = type.m_vertices;
        for (int j = 0; j < H.m_vertices; j++)  if (vertex_split[j] == -2) mapping[next_to_map++] = j;
        tmp.as_subflag(H,mapping,F2.m_vertices,type.m_Theta);
        if (!tmp.is_isomorphic_to(F2)) continue;


        
        // check if F3 correct
        next_to_map = type.m_vertices;
        for (int j = 0; j < H.m_vertices; j++)  if (vertex_split[j] == -3) mapping[next_to_map++] = j;
        tmp.as_subflag(H,mapping,F3.m_vertices,type.m_Theta);
        if (!tmp.is_isomorphic_to(F3)) continue;
        
        good_maps++;
        
    } while ( std::next_permutation(vertex_split,vertex_split+H.m_vertices) ); 

    //cerr << "Good/All: " << good_maps << "/" << all_maps << endl;
    
    return (double)good_maps/(double)all_maps;
}




inline double P_F1_IN_H(const flag &F1, const flag &H, bool density=true)
{
    
    assert(F1.m_vertices <= H.m_vertices);
    
    flag type;
    F1.get_type_subflag(type);    

    if (H.labeled_vertices_cnt() > 0)
    {
        flag typeH;
        H.get_type_subflag(typeH);    
        if (!type.contains_as_subflag(typeH)) return 0;
    }    
    
    // Some easy checks...
#if defined(G_COLORED_EDGES) && !defined(G_COLORED_EDGES_BLIND)
        
    for (int c = 1; c < COLORS_EDGES; c++)
    {
        if (F1.m_colored_edges[c] > H.m_colored_edges[c]) return 0;
    }
#endif
       
    
    int good_maps = 0;
    int all_maps = 0;
    
    int vertex_split[V];
    
    int t = 0;
    for (int i = 0; i < type.m_vertices; i++) vertex_split[t++] = i;
    for (int i = type.m_vertices; i < F1.m_vertices; i++) vertex_split[t++] = -1;
    for (;t < H.m_vertices; t++) vertex_split[t] = -4;
    
    std::sort (vertex_split+H.labeled_vertices_cnt(),vertex_split+H.m_vertices);
    
    flag tmp;
    int mapping[V];
    int next_to_map; // temporary
    do {
        all_maps++;
        
        
        // check if Theta correct
        for (int j = 0; j < H.m_vertices; j++)
        {
            if (vertex_split[j] < 0) continue;
            if (vertex_split[j] < type.m_vertices) mapping[vertex_split[j]] = j;
        }
        tmp.as_subflag(H,mapping,type.m_vertices,type.m_Theta);
        //cerr << tmp.print() << " vs " << type.print();
        if (!tmp.is_isomorphic_to(type)) continue;
        
        
        // check if F1 correct
        next_to_map = type.m_vertices;
        for (int j = 0; j < H.m_vertices; j++)  if (vertex_split[j] == -1) mapping[next_to_map++] = j;
        tmp.as_subflag(H,mapping,F1.m_vertices,type.m_Theta);
        if (!tmp.is_isomorphic_to(F1)) continue;
        
        good_maps++;
        
    } while ( std::next_permutation(vertex_split+H.labeled_vertices_cnt(),vertex_split+H.m_vertices) ); 
    
    //cerr << "Good/All: " << good_maps << "/" << all_maps << endl;
    
    if (density)
        return (double)good_maps/(double)all_maps;
    else
        return good_maps;
}



bool load_flags_from_file(string filename, vector<flag> &flag_list,bool verbose_output=false)
{
    ifstream infile;
    infile.open (filename.c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file with flags " << filename << endl;
        return false;
    }
    
    if (verbose_output)
        cerr << "Loading labeled flags from file " << filename << endl;
    
    
    flag f;
    while (f.load_from_stream(infile,-1,-1))
    {
        flag_list.push_back(f);
    }
    
    infile.close();
    
    return true;    
}

bool dump_flags_to_file(string filename, vector<flag> &flag_list)
{
    ofstream outfile;
    outfile.open(filename.c_str(), ofstream::out);
    if (!outfile.good())
    {
        cerr << "Failed opening file " << filename << endl;
        return false;
    }
    
    cerr << "Writing flags to file " << filename << endl;
    
    for (unsigned int x = 0; x < flag_list.size(); x++)
    {
        outfile << flag_list[x].print() << endl;
    }
    
    outfile.close();
    
    return true;
}

bool load_flags_and_coefficients_from_file(string filename, vector<flag_and_coefficient> &flag_list, bool verbose_output=false)
{
    istream *istr = &cin;
    
    ifstream infile;
    if (filename != "cin")
    {
        infile.open (filename.c_str(), ifstream::in);
        if (!infile.good())
        {
            cerr << "Failed opening file " << filename << endl;
            return false;
        }
        istr = &infile;
    }
    
    if (verbose_output)
        cerr << "Loading labeled flags from file " << filename << endl;
    
    
    flag_and_coefficient fc;
    (*istr) >> fc.coefficient;
    while ((*istr) && fc.g.load_from_stream((*istr),-1,-1))
    {
        flag_list.push_back(fc);
        (*istr) >> fc.coefficient;
    }
    
    infile.close();
    
    return true;
}

void dump_flags_and_coefficients(vector<flag_and_coefficient> &flag_list)
{
    for (unsigned int x = 0; x < flag_list.size(); x++)
    {
        cout.precision(17);
        cout << flag_list[x].coefficient << "  " << flag_list[x].g.print() << endl;
    }
}



// final_flags means that the flags will not be used any further
// for genertin other flags. Important when combined with G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY
bool load_unlabeled_flags_from_file(int sizeKn, bool final_flags = true, bool remove_duplicates_while_loading=false)
{
    stringstream filename;
    
    filename << filename_prefix() << "__n" << sizeKn << "_unlabeled.txt";
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file with unlabeled flags " << filename.str() << endl;
        return false;
    }
    
    cerr << "Loading unlabeled flags from file " << filename.str() << endl;
    
    // For large instances, it really helps to reserve the space in containers
    //  before loading... so we read the file twice
    int sizes_cnt[V], duplicates_cnt[V];
    for (int i = 0; i < V; i++)
    {
        sizes_cnt[i] = 0;
        duplicates_cnt[i] = 0;
    }
    flag g;
    while (g.load_from_stream(infile,-1,0))
    {
        sizes_cnt[g.m_vertices]++;
    }
    for (int i = 0; i < V; i++) g_unlabeled_flags[i].reserve(sizes_cnt[i]);
    
    //    infile.seek(0);
    infile.close();
    infile.open (filename.str().c_str(), ifstream::in);
    
    
    while (g.load_from_stream(infile,-1,0))
    {
        
        assert(sizeKn >= g.m_vertices);
        
#ifdef G_CHECK_FOR_FORBIDDEN_WHILE_LOADING_FLAGS
        if (is_flag_forbidden(g))
        {
            continue;
        }
#endif
        if (remove_duplicates_while_loading)
        {
            //#ifdef G_REMOVE_DUPLICATES_WHILE_LOADING_FLAGS
            bool new_flag = true;
            for (int i = 0; i < (int)g_unlabeled_flags[g.m_vertices].size();i++)
            {
                if (g.is_isomorphic_to(g_unlabeled_flags[g.m_vertices][i]))
                {
                    new_flag = false;
                }
            }
            if (!new_flag)
            {
                duplicates_cnt[g.m_vertices]++;
                continue;
            }
            //#endif
        }
        g_unlabeled_flags[g.m_vertices].push_back(g);
    }
    
    infile.close();
    
    for (int i = 0; i <= sizeKn; i++)
    {
        cerr << "Loaded # of unlabeled flags of size " << i << " is " << g_unlabeled_flags[i].size();
        if (remove_duplicates_while_loading)
        {
            cerr << " and " << duplicates_cnt[i] << " duplicates";
        }
        cerr << endl;
    }
    
    
    return true;
}



bool dump_unlabeled_flags(int sizeKn)
{
    stringstream filename;
    filename << filename_prefix() << "__n" << sizeKn << "_unlabeled.txt";
    
    ofstream outfile;
    outfile.open (filename.str().c_str(), ofstream::out);
    if (!outfile.good())
    {
        cerr << "Failed opening file " << filename.str() << endl;
        return false;
    }
    
    cerr << "Writing unlabeled flags to file " << filename.str() << endl;
    
    for (int f = 0; f < V; f++)
    {
        for (unsigned int x = 0; x < g_unlabeled_flags[f].size(); x++)
        {
            outfile << g_unlabeled_flags[f][x].print() << endl;
        }
    }
    
    outfile.close();
    
    return true;
}




void try_extensions_of_g_to_last_vertex_edges(flag &g, vector<flag> &flag_list)
{
    
    int v = g.m_vertices;
#ifdef G_COLORED_EDGES
    try_color_edge(g,v-1,0,flag_list);
    return;
#endif
    
    assert(0);
}


void try_extensions_of_g_to_last_vertex(flag &g, vector<flag> &flag_list)
{
    try_extensions_of_g_to_last_vertex_edges(g, flag_list);
}


void dump_vector_of_flags(vector <flag> *flag_lists, int previous_size, int sizeKn)
{
    stringstream filename;
    filename << filename_prefix() << "__n" << sizeKn << "_unlabeled_dump.txt";
    
    ofstream outfile;
    outfile.open (filename.str().c_str(), ofstream::out);
    if (!outfile.good())
    {
        cerr << "Failed opening file " << filename.str() << endl;
        return;
    }
    
    // dumping of flag lists...
    for (int i = 0; i < previous_size; i++)
    {
        for (int f = 0; f < (int)flag_lists[i].size(); f++)
        {
            outfile << flag_lists[i][f].print() << endl;
        }
        outfile << endl;
    }
    
    outfile.close();
}



// The following function megres dest_flags with src_flags and clears src_flags
void merge_flags_lists_parallel(vector<flag> &dest_flags, vector<flag> &src_flags)
{
    
    //    cout << flag_list.size() << endl;
    
    vector<flag> new_flags;
    new_flags.reserve(src_flags.size());
    
#pragma omp parallel for
    for (int k = 0; k < (int)src_flags.size(); k++)
    {
        if (!g_already_in_known_flags(src_flags[k],dest_flags))
        {
#pragma omp critical (merging_lists)
            {
                new_flags.push_back(src_flags[k]);
            }
        }
    }
    //#ifdef DONT_USE_C11
    dest_flags.insert(dest_flags.end(), new_flags.begin(), new_flags.end() );
    src_flags.clear();
    vector<flag>().swap( src_flags ); // free the memory
    //#else
    //    dest_flags.insert(dest_flags.end(), std::make_move_iterator(new_flags.begin()), std::make_move_iterator(new_flags.end()) );
    //    src_flags.clear();
    //    src_flags.shrink_to_fit(); // free the memory - does not work well on icc
    //#endif  
}



void generate_unlabeled_flags_of_size(int i, bool verbose_output, bool dump_unlabeled_while_generating)
{
    
    
    int previous_size = (int)g_unlabeled_flags[i-1].size();
    
    //vector <flag> flag_lists[previous_size]; // does not work on mac (llvm :-( )
    vector <flag> *flag_lists;
    flag_lists = new vector<flag>[previous_size];
    if (flag_lists == NULL)
    {
        cerr << "Memory allocation failed" << endl;
        exit(1);
    }
    
    //            #pragma omp parallel for
#pragma omp parallel for ordered schedule(dynamic)
    for (int j = 0; j < (int)g_unlabeled_flags[i-1].size(); j++)
    {
        flag_lists[j].reserve(previous_size);
        
        flag g;
        g.set_vertices(i);
        g.copy_from(g_unlabeled_flags[i-1][j]);
        //vector <flag> valid_extensions_of_g;
        //cout << "going for " << j << endl;
        try_extensions_of_g_to_last_vertex(g, flag_lists[j]);
        if (verbose_output)
        {
            cerr << "generated size " << i << ": " << j  << "/" << (int)g_unlabeled_flags[i-1].size() << " found " << flag_lists[j].size() << " flags " << endl;
        }
    }
    if (verbose_output)
    {
        cerr << "Flags generated. Merging part starts." << endl;
    }
    
    if (dump_unlabeled_while_generating)
        dump_vector_of_flags(flag_lists, previous_size, i);
    
    for (int power = 1; power < previous_size; power *= 2)
    {
        if (verbose_output)
        {
            cerr << "Trying " << power << " in " << previous_size << endl;
        }
        for (int j = 0; j < previous_size; j += 2*power)
        {
            if (j+power < previous_size)
            {
                if (verbose_output)
                {
                    cerr << "Merging  " << j << "<-" << j+power << "  of sizes " <<  flag_lists[j].size() << " " << flag_lists[j+power].size() << endl;
                }
                merge_flags_lists_parallel(flag_lists[j],flag_lists[j+power]);
                //flag_lists[j+power].clear();
            }
        }
        if (dump_unlabeled_while_generating)
            dump_vector_of_flags(flag_lists, previous_size, i);
    }
    
    g_unlabeled_flags[i].swap(flag_lists[0]);
    
    delete[] flag_lists;
}


void get_unlabeled_flags_of_size(int Kn, bool force_generate_flags = false, bool remove_duplicates_while_loading = false, bool verbose_output = false, bool dump_unlabeled_while_generating= false)
{
    // Already loaded
    if (g_unlabeled_flags[Kn].size() > 0)
    {
        return;
    }
    
    // Getting unlabeled flags...
    if (force_generate_flags || !load_unlabeled_flags_from_file(Kn,true,remove_duplicates_while_loading))
    {
        // Try to reuse previous graphs...
        int loaded_size = 0;
        
        // blow-upping and generating would not work correctly...
        if (!force_generate_flags)
        {
            loaded_size = Kn-1;
            cerr << "Trying to load file with smaller flags..." << endl;
            while (loaded_size > 0 && !load_unlabeled_flags_from_file(loaded_size,false)) loaded_size--;
        }
        
        if (loaded_size == 0)
        {
            cerr << "Unable to load any smaller graphs. Starting from empty graph." << endl;
            flag g_zero;
            g_zero.set_vertices_and_Theta(0,0);
            g_unlabeled_flags[0].push_back(g_zero);
        }
        
        cerr << "Generating unlabeled flags of sizes " << loaded_size+1 << " to " << Kn << endl;
        
        for (int i = loaded_size+1; i <= Kn; i++)
        {
            generate_unlabeled_flags_of_size(i, verbose_output, dump_unlabeled_while_generating);
            cerr << "Unlabbeled flags of size "<< i << ": " << g_unlabeled_flags[i].size() << endl;
        }
        
        
        dump_unlabeled_flags(Kn);
    }
    
}





    
// f is an unlabeled flag
void count_flag_products(stringstream &ss, int matrixID, const flag &f)
{    
    int subset_split[V];
    int mapping_F1[V];
    int mapping_F2[V];
    flag F1, F2; 
 
    // Try to find decomposition of vertices of f into 3 sets.
    // first is of size typesize and the other two are of sizes privatesize
    // Then typesize+privatesize creates one flag.
    
    // We go through all possible type sizes
    for (int typesize = f.m_vertices-2; typesize >= 0; typesize -= 2)
    {
        vector<pair<string,int> > found_pairs;

        int privatesize = (f.m_vertices-typesize)/2; 
        int totalsize = typesize+privatesize;  // size of F1 and F2
        int label_f1 = typesize;
        int label_f2 = typesize+1;

        for (int i = 0; i < typesize; i++) subset_split[i] = i;
        assert(typesize+2*privatesize < V);
        for (int i = 0; i < privatesize; i++)
        {   
            subset_split[typesize+i] = label_f1;
            subset_split[typesize+i+privatesize] = label_f2; // I don't know why the warning is here...
        }
        
        do {
            //std::cout << subset_split[0] << ' ' << subset_split[1] << ' ' << subset_split[2] << ' ' << subset_split[3] << '\n';
        
            // Construction of inverz mapping
            int inF1 = typesize;
            int inF2 = typesize;            
            for (int i = 0; i < f.m_vertices; i++)
            {
                if (subset_split[i] < typesize) 
                {
                    mapping_F1[subset_split[i]] = i;
                    mapping_F2[subset_split[i]] = i;
                }
                if (subset_split[i] == label_f1) mapping_F1[inF1++] = i; 
                if (subset_split[i] == label_f2) mapping_F2[inF2++] = i; 
            }

            F1.as_subflag(f,mapping_F1, totalsize, typesize);
        
            int typeID = get_flag_type_in_list(F1, g_flags);
            if (typeID == -2) continue;
#ifdef G_NOT_ALL_FLAGS_USED
            if (typeID == -1) continue;
#endif            
            assert (typeID != -1);
            
            F2.as_subflag(f,mapping_F2, totalsize, typesize);

            int idF1 = find_flag_in_list(F1,g_flags[typeID]);
            int idF2 = find_flag_in_list(F2,g_flags[typeID]);
            if (idF1 > idF2 ) continue; // count only once when idF1 <= idF2         
            if (idF1 == -1)
            {
#ifdef G_NOT_ALL_FLAGS_USED
                continue;
#endif                
                cerr << "Missing " << F1.print() << endl;
            }
            assert(idF1 >= 0 && idF2 >= 0); 
            
            stringstream what_found_ss;
            what_found_ss << typeID+1 << " " << idF1+1 << " " << idF2+1;
            string what_found = what_found_ss.str();
            
            bool known = false;
            for(int i = 0; i < (int)found_pairs.size(); i++)
            {
                if (found_pairs[i].first == what_found) 
                {
                    found_pairs[i].second++;
                    known = true;
                    break;
                }
            }
            if (known == false)
            {
                found_pairs.push_back(make_pair(what_found,1));
            }
            
        } while ( std::next_permutation(subset_split,subset_split+f.m_vertices) );
        
        for(int i = 0; i < (int)found_pairs.size(); i++)
        {
            ss << matrixID << " " << found_pairs[i].first << " " <<  found_pairs[i].second << endl; 
        }
    }
}


double compute_objective_function_for_SDP(const flag &g)
{

	double total_density = 0;	
#ifdef USE_COUNT_INSTEAD_OF_DENSITY_IN_OBJECTIVE
	for (int i = 0; i < (int)g_objective_combination.size(); i++)
	{
		//total_density += g.count_subflag(g_objective_combination[i].g)*g_objective_combination[i].coefficient;
        total_density += g_objective_combination[i].coefficient * P_F1_IN_H(g_objective_combination[i].g, g);
	}	
	return total_density;	
#endif

	for (int i = 0; i < (int)g_objective_combination.size(); i++)
	{
//		total_density += g.density_subflag(g_objective_combination[i].g)*g_objective_combination[i].coefficient;
        total_density += g_objective_combination[i].coefficient * P_F1_IN_H(g_objective_combination[i].g, g);
	}
	
	return total_density;
}


int print_CSDP_simple_linear_constraints(ostream &ostr, int Kn, int i, int matrixID, int blockID, bool print_blocks, bool print_products, bool verbose_output)
{
    
    //assert(0);

    return 0;
}




bool has_slack(int Kn, int i)
{
    if (find_flag_in_list(g_unlabeled_flags[Kn][i],g_no_slack_flags) != -1)
    {
        cerr << "Flag " << i << " has no slack" << endl;
    }
    return (find_flag_in_list(g_unlabeled_flags[Kn][i],g_no_slack_flags) == -1);
}


// According to http://plato.asu.edu/ftp/sdpa_format.txt
//
// Solved program
//   max C.X
//   subject to A_1.X = b_1
//              A_2.X = b_2
//              A_mdim.X = b_mdim
//   where X is positive semidefinite matrix
//
// In our UPPER BOUND application
//
//      min    t
//      s.t.  D_i + [A.X]_i <= t
//
// We add a slack variable s_i to make the <= to = and move D_i (density - constant to the other side)
// and change to maximization:
//
//      max   -t
//      s.t.  [A.X]_i + s_i - t = -D_i
//
//   C.X is just entry 0 2 1 1 -1.0 which means that in the second block,
//   the first variable is with -1. Se we are maximizing -t which corresponds to minimizing t
//
// Now LOWER BOUND application
//
//      max    t
//      s.t.  D_i - [A.X]_i  >= t
//
// We add a slack variable s_i to make the >= to = and move D_i (density - constant to the other side)
// and change to maximization:
//
//      max   t
//      s.t.  [A.X]_i + s_i + t = D_i
//
// Finally, Additional constraints applications:
// Say we want to add  a*G >= c. Then we could add
//  k(a*G(sampled in H_i) -c) >= 0
//
// For lower bound we get (max t)
//     [A.X]_i + k*(a*G_i-c) + s_i + t = D_i
//
// For upper bound we get (min t)
//     [A.X]_i + k*(a*G_i-c) + s_i -t = D_i
//
//
void print_CSDP_specific_part(int Kn, bool upper_bound = true, ostream &ostr = cout, bool verbose_output = true)
{
    int mdim = 0;
    mdim = (int)g_unlabeled_flags[Kn].size();
    
    ostr << mdim << endl; //<<" =mdim" << endl; // number of constraint matrices
    
    stringstream ssblocks; // String stream for all blocks in the matrix
    int blocks = 0; // number of blocks in ssblocks
 
    // First blocks are flags products:
    for (int f = 0; f < (int)g_flags.size(); f++) ssblocks <<  g_flags[f].size() << " ";
    blocks = (int)g_flags.size();
    
    // next we have one diagonal block of slack variables
    ssblocks << -1-mdim << " ";  // get_number_of_slacks() but we don't go for it here...
    blocks++;

    // next we have linear constraints blocks
    int block_ID_simple_linear_constraints = -1;
    if (g_linear_constraints.size() != 0)
    {
        if (g_use_simple_linear_constraints)
        {
        // Simple linear constraints
        int simple_constraints = print_CSDP_simple_linear_constraints(ssblocks, Kn, 0, 0, 0, true, false, verbose_output);
        if (simple_constraints != 0)
        {
            block_ID_simple_linear_constraints = blocks+1;
            blocks++;
        }
        }
        
    }


    ostr << blocks << endl;
    ostr << ssblocks.str() << endl;
 
    for (unsigned int i = 0; i < g_unlabeled_flags[Kn].size(); i++)
    {
        ostr.precision(17);
        if (upper_bound) ostr << -1*smart_round(compute_objective_function_for_SDP(g_unlabeled_flags[Kn][i])) << " "; // outputing b_i
        else             ostr << smart_round(compute_objective_function_for_SDP(g_unlabeled_flags[Kn][i])) << " "; // outputing b_i
    }
    ostr << endl;
    
    if (upper_bound) ostr << "0 " << (int)g_flags.size()+1 << " 1 1 -1" << endl; // Objective function
    else             ostr << "0 " << (int)g_flags.size()+1 << " 1 1 1" << endl; // Objective function


    // printing of subflags
#pragma omp parallel for ordered schedule(dynamic)
        for (int i = 0; i < (int)g_unlabeled_flags[Kn].size(); i++)
        {            
            //cout << "flag: "; c4free_subrgaphs[e][i].print();
            if (verbose_output) cerr << "Computing specific part " << i+1 << "/" << g_unlabeled_flags[Kn].size() << endl;
            stringstream ss;
            
            int matrixID = i + 1;
         
            // variable for objective function
            if (upper_bound) ss << matrixID << " " << (int)g_flags.size()+1 << " 1 1 -1" << endl;
            else             ss << matrixID << " " << (int)g_flags.size()+1 << " 1 1 1" << endl;
            
            // variable for slack
            if (has_slack(Kn,i))
            {
                ss << matrixID << " " << (int)g_flags.size()+1 << " " << matrixID+1 << " " <<  matrixID+1 << " 1" << endl;
            }
            
            if (g_use_simple_linear_constraints)
            {
                print_CSDP_simple_linear_constraints(ss, Kn, i, matrixID, block_ID_simple_linear_constraints, false, true, verbose_output);
            }
            
        
#pragma omp ordered
{
            ostr << ss.str();
}
        }
    ostr.flush();
    
}

void print_CSDP_flag_products_part(int Kn, bool upper_bound = true, ostream &ostr = cout, bool verbose_output = true, int lowest_not_computed=0)
{
#pragma omp parallel for ordered schedule(dynamic)
    for (int i = lowest_not_computed; i < (int)g_unlabeled_flags[Kn].size(); i++)
    {            
        if (verbose_output) cerr << "Computing flag products " << i+1 << "/" << g_unlabeled_flags[Kn].size() << endl;

        stringstream ss;
        
        int matrixID = i + 1;
        
#ifdef G_FLAG_PRODUCTS_SLOW_LOW_MEMORY       
         for (int f = 0; f < (int)g_flags.size(); f++)
         {
             for (unsigned int x = 0; x < g_flags[f].size(); x++)
                 for (unsigned int y = x; y < g_flags[f].size(); y++)
                 {
         
                     double PF = P_F1_F2_IN_H(g_flags[f][x], g_flags[f][y], g_unlabeled_flags[Kn][i]);
                     //					if (x == y) PF = PF*0.5;
                     if (PF != 0)
                     {
                         ss << matrixID <<  " " << f+1 << " " << x+1 << " " << y+1 << " " <<  PF << endl;
                     }
                 }
         }
#else
        count_flag_products(ss, matrixID, g_unlabeled_flags[Kn][i]);
#endif        
        		
#pragma omp ordered
        {
            ostr << ss.str();
        }
    }
    ostr.flush();
}

void print_CSDP(int Kn, bool upper_bound = true, ostream &ostr = cout, bool verbose_output = true, bool use_sdp_temp=false)
{
    cerr << "Computing specific part of the SDP..." << endl;
    print_CSDP_specific_part(Kn, upper_bound, ostr, verbose_output);
    
    if (!use_sdp_temp)
    {
        cerr << "Computing flag products part of the SDP..." << endl;
        print_CSDP_flag_products_part(Kn, upper_bound, ostr, verbose_output);
        return;
    }
    
    cerr << "Getting flag products for SDP..." << endl;    
    
    stringstream filename;
    
    filename << filename_prefix() << "__n" << Kn << "_sdp_products.txt";

    // The sdp_product_file may not contain all products if the computation
    // was accidentally interrupted. We try to go from we stopped...
    // First find were we stopped
    
    int lowest_not_computed = 0;

    ifstream sdp_product_file;
    sdp_product_file.open (filename.str().c_str(),ifstream::in);
    while (sdp_product_file.good())
    {
        int id, type, f1, f2, d;
        sdp_product_file >> id >> type >> f1 >> f2 >> d;
        if (!sdp_product_file.good()) break; // if the read failed, do not use it...
        if (id > lowest_not_computed)
        {
            lowest_not_computed = id; // note that in file the ID is +1
        }
    }
    sdp_product_file.close();
    
    if (lowest_not_computed < (int)g_unlabeled_flags[Kn].size())
    {
        if (lowest_not_computed != 0)
        {
            cerr << "Warning: file " << filename.str() << " contains only products up to " << lowest_not_computed << " out of " << (int)g_unlabeled_flags[Kn].size() << endl;
            cerr << "          We assume it happened by accident so the rows starting with " << lowest_not_computed << " will be deleted and recomputed" << endl;
            stringstream mv_command;
            mv_command << "rm -f "<< filename.str() << "~ ;  mv " << filename.str() << " " << filename.str() << "~";
            cerr << "          Executing " << mv_command.str() << endl;
            if (system(mv_command.str().c_str()) != 0)
            {
                cerr << " Execution failed" << endl;
                exit(1);
            }
            // We remove the last line since it may be incoplete and not catched by the grep
            stringstream grep_command;
            grep_command << "sed \\$d " << filename.str() << "~ |   grep -v '^" << lowest_not_computed << " ' "  << " >" << filename.str();
            cerr << "          Executing " << grep_command.str() << endl;
            if (system(grep_command.str().c_str()) != 0)
            {
                cerr << " Execution failed" << endl;
                exit(1);
            }
            lowest_not_computed--;
        }
     
        ofstream sdp_product_file_out;
        sdp_product_file_out.open (filename.str().c_str(),ofstream::out|ofstream::app);
        if (!sdp_product_file_out.good())
        {
            cerr << "Failed creating file with sdp products " << filename.str() << endl;
            exit(1);
        }
        
        cerr << "Computing flag products of the SDP to file " << filename.str() << endl;
        print_CSDP_flag_products_part(Kn, upper_bound, sdp_product_file_out, verbose_output, lowest_not_computed);
        sdp_product_file_out.close();
    }

   
    sdp_product_file.open (filename.str().c_str(),ifstream::in);
    if (!sdp_product_file.good())
    {
        cerr << "Failed obtaining file with sdp products " << filename.str() << endl;
        exit(1);
    }
    
    cerr << "Copying flag products of the SDP from " << filename.str() << endl;
    ostr << sdp_product_file.rdbuf();
    sdp_product_file.close();
}








// Not paralel version - very slow and not completely correct - in particular for ordered version.
void generate_labeled_flags_old(int flag_size, int type_size, bool verbose_output)
{
    int to_label_size = (int)g_unlabeled_flags[flag_size].size();
    //cerr << "Type size " << type_size << " for " << g_unlabeled_flags[flag_size][i].print() << endl;
    int mapping[flag_size];
    flag F;
    
    for (int i = 0; i < to_label_size; i++)
    {
        if (verbose_output)
            cerr << "Flag size "<<flag_size << " type size " << type_size << " labeling " << i << "/" << to_label_size << endl;

        for (int j = 0; j < flag_size; j++) mapping[j]=j;
        
        do {
            F.as_subflag(g_unlabeled_flags[flag_size][i], mapping, flag_size, type_size); // taking a subflag
            include_flag_in_list_if_new(F,g_flags);
        } while ( std::next_permutation(mapping,mapping+flag_size) );
    }
}


//vector<flag> g_types[V];
void create_types_of_size(int type_size)
{
    if (g_types[type_size].size() != 0) return;


    // Look if there are already some flags with given types...
    for (int i = 0; i < (int)g_flags.size(); i++)
    {
        if (g_flags[i].size() == 0) continue;
        
        if (g_flags[i][0].labeled_vertices_cnt() == type_size)
        {
            flag type;
            g_flags[i][0].get_type_subflag(type);
            g_types[type_size].push_back(type);
        }
    }
    if (g_types[type_size].size() != 0) return;
    
    
    // if not, just label graphs of apropriate size
    get_unlabeled_flags_of_size(type_size);
    
    g_types[type_size] = g_unlabeled_flags[type_size];

    
#pragma omp parallel for
    for (int i = 0; i < (int)g_types[type_size].size(); i++)
    {
        g_types[type_size][i].make_all_vertices_labeled();
        //
        cerr << "New type: " << g_types[type_size][i].print() << endl;
    }
    
    
    cerr << "Generated " << g_types[type_size].size()  << " types of size " << type_size << endl;
}

int get_reduced_type_id(const flag &f)
{
    int type_size = f.labeled_vertices_cnt();
    
    //cerr << "Testing size " << type_size << endl;
    
    for (int i = 0; i < (int)g_types[type_size].size(); i++)
    {
        if (f.have_same_type(g_types[type_size][i]))
        {
            return i;
        }
    }    
    return -1;
}


void get_types_of_size(int type_size, bool types_from_file)
{
//    create_types_of_size(type_size);
//    return;
    
    if (!types_from_file)
    {
        create_types_of_size(type_size);
        return;
    }
    
    string filename = filename_prefix() + "__type_"+ to_string(type_size) +".txt";
    
    if (!load_flags_from_file(filename, g_types[type_size]))
    {
        create_types_of_size(type_size);
        dump_flags_to_file(filename, g_types[type_size]);
    }
    else
    {
        cerr << "Types loaded from file " << filename << endl;
    }
}


//
void generate_labeled_flags(int flag_size, int type_size, bool verbose_output, bool types_from_file=false)
{
    get_types_of_size(type_size, types_from_file);
    //cerr << "B:EEEE  " << g_types[type_size].size() << endl;
    //cerr <<  "flag_size=" << flag_size << " type_size=" << type_size << endl;
    
    int to_label_size = (int)g_unlabeled_flags[flag_size].size();
    
    // 
    vector< vector<flag> >  *flags;
    flags = new vector< vector<flag> >[to_label_size];
    
    int types = (int)g_types[type_size].size();
    
    // Avoid copying flags
    for (int i = 0; i < to_label_size; i++)
    {
        flags[i].resize(types);
    }
    
    cerr << "Generating labeled flags for Cauchy-Swartz in parallel " << endl;
#pragma omp parallel for ordered
    for (int i = 0; i < to_label_size; i++)
    {
        if (verbose_output)
            cerr << "Flag size "<<flag_size << " type size " << type_size << " labeling " << i << "/" << to_label_size << endl;

        //cerr << "Type size " << type_size << " for " << g_unlabeled_flags[flag_size][i].print() << endl;
        // Permutations for labeling... Could be improved to distinguish only the first type_size vertices.
        int mapping[flag_size];
        flag F;
        for (int j = 0; j < flag_size; j++) mapping[j]=j;
        
        //int found_good_type = 0;
        do {
            F.as_subflag(g_unlabeled_flags[flag_size][i], mapping, flag_size, type_size); // taking a subflag
            //cerr << "Got one" << endl;
            int type_of_F = get_reduced_type_id(F);
            if (type_of_F == -1)
            {
                // We have each type only once -  3 3   1 1  2  and  3 3   1 2  1 are different types but enough to use just one
                continue;
            }
            //found_good_type++;
            if (!g_already_in_known_flags(F,flags[i][type_of_F]))
            {
                    flags[i][type_of_F].push_back(F);
            }
            //include_flag_in_list_if_new(F,flags[i],false);
        } while ( std::next_permutation(mapping,mapping+flag_size) );
        //cout << "Found good type " << found_good_type << endl;
    }
    
    // Todo: this might be also maybe paralelized
    for (int i = 0; i < to_label_size; i++)
    {
        for (int j = 0; j < (int)flags[i].size(); j++)
            for (int k = 0; k < (int)flags[i][j].size(); k++)
                include_flag_in_list_if_new(flags[i][j][k], g_flags);
        
        //merge_typed_flags_lists_parallel(g_flags, flags[i]);
    }
  
    delete[] flags;
}


//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP

void load_linear_constraints_from_stream(istream &instream, bool verbose_output)
{
    //
    flag_and_coefficient fc;
    string lc_constat;
        
    while (instream)
    {
        linear_constraint lc;
        instream >> lc.m_constant;
        if(!instream.good()) {
            // Not a number means comment continues...
            std::istreambuf_iterator<char> eos;
            std::string s(std::istreambuf_iterator<char>(instream), eos);
            g_program_description = s;
            break;
        }
        instream >> fc.coefficient;
        
        if (verbose_output)
            cerr << "Loading linear constraint 0 <= " << lc.m_constant <<  endl;
        
        assert (fc.coefficient != 0);
        while (instream && fc.g.load_from_stream(instream,-1,-1))
        {
            if (verbose_output)
                cerr << "..+..  " << fc.coefficient << " * " << fc.g.print() << endl;
            
            lc.m_entries.push_back(fc);
            instream >> fc.coefficient;

            if (fc.coefficient == 0)
            {
                break;
            }
        }   
        
        lc.check_constraint();
        g_linear_constraints.push_back(lc);  
    }
}


bool load_linear_constraints_from_file(const string &filename, bool verbose_output)
{
    ifstream infile;
    infile.open (filename.c_str(), ifstream::in);
    if (infile.good())
    {
    
        cerr << "Loading additional constraints from file " << filename  <<  endl;
        int tmp;
        infile >> tmp;
        assert(tmp == 0);
        load_linear_constraints_from_stream(infile, verbose_output);
        infile.close();
        return true;
    }
    return false;
}


bool load_objective_from_file(const string &filename, bool verbose_output)
{
    ifstream infile;
    infile.open (filename.c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file with objective function " << filename << endl;
        return false;
    }
    
    cerr << "Loading objective function from file " << filename << endl;
        
    flag_and_coefficient fc;
    
    infile >> fc.coefficient;
    assert (fc.coefficient != 0);
    while (infile && fc.g.load_from_stream(infile,-1,-1))
    {
        // check if the flag is labeled
        if (fc.g.labeled_vertices_cnt() != 0)
        {
            cerr << "WARNING: You are using labeled flags in th objective function" << endl;
        }
        // check if there are no repetitions in the objective function..
        for (int i = 0; i < (int)g_objective_combination.size(); i++)
        {
            if (g_objective_combination[i].g.is_isomorphic_to(fc.g))
            {
                cerr << "WARNING: In the objecive function, flags " << g_objective_combination[i].g.print() << " and " <<
                fc.g.print() << " are the same " << endl;
                //exit(1);
            }
        }
        g_objective_combination.push_back(fc);
        infile >> fc.coefficient;
        
        if (fc.coefficient == 0)
        {
            load_linear_constraints_from_stream(infile, verbose_output);
            break;
        }
    }
    
    infile.close();
    
    cerr << "# of entries in objective is " << g_objective_combination.size() << endl;
    cerr << "# of linear constraints is " << g_linear_constraints.size() << endl;
    
    return true;
}





bool load_labeled_flags_from_file(int sizeKn, bool verbose_output = false)
{
    stringstream filename;
    filename << filename_prefix() << "__n" << sizeKn << "_labeled.txt";
    
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file with labeled flags " << filename.str() << endl;
        return false;
    }

    if (verbose_output)
    {
        cerr << "Loading labeled flags from file " << filename.str() << endl;
    }

    
   flag f;
	while (f.load_from_stream(infile,-1,-1))
	{
		include_flag_in_list(f, g_flags);
	}
	
	infile.close();
    
	return true;
}

bool dump_labeled_flags(int sizeKn)
{
    stringstream filename;
    filename << filename_prefix() << "__n" << sizeKn << "_labeled.txt";
    
    ofstream outfile;
    outfile.open (filename.str().c_str(), ofstream::out);
    if (!outfile.good())
    {
        cerr << "Failed opening file " << filename.str() << endl;
        return false;
    }
    
    cerr << "Writing labeled flags to file " << filename.str() << endl;
    
    for (int f = 0; f < (int)g_flags.size(); f++)
    {
        for (unsigned int x = 0; x < g_flags[f].size(); x++)
        {
            outfile << g_flags[f][x].print() << endl;
        }
        if (f < (int)g_flags.size()-1) outfile << endl;
    }
    
    outfile.close();
    
    return true;
}



bool compare_flag_sizes(const flag&f1, const flag &f2)
{
    return f1.m_vertices < f2.m_vertices;
}

void sort_flags_by_size(vector<flag> &flag_list)
{
    sort(flag_list.begin(), flag_list.end(), compare_flag_sizes);
}



void load_forbidden()
{
    stringstream filename;
    filename <<  filename_prefix() << "__forbidden.txt";
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file " << filename.str() << " no forbidden structures are used" << endl;
        return;
    }
    
    int flags_in_file = 0;
    
    flag h;
    while (h.load_from_stream(infile,-1,0))
    {
        flags_in_file++;
        // check if h is not already there
        if (!(g_already_in_known_flags(h,g_forbidden_subflags)))
        {
            g_forbidden_subflags.push_back(h);
            g_forbidden_subflags_by_size[h.m_vertices].push_back(h);
            //cerr << "Loaded forbidden graph " << h.print() << endl;
        }
    }
    
    infile.close();
    
    // Idea is that it is faster to test smaller subflags than bigger
    // so lets first test smaller ones
    sort_flags_by_size(g_forbidden_subflags);
    
    cerr << "Loaded " << g_forbidden_subflags.size() << " forbidden graphs from " << filename.str() << endl;
    if (flags_in_file != (int)g_forbidden_subflags.size())
    {
        cerr << "File " << filename.str() << " contains " << flags_in_file-g_forbidden_subflags.size() << " duplicate  graphs." << endl;    
    }
    
    //for (int i = 0; i < g_forbidden_subflags.size(); i++)
    //{
    //    cerr << g_forbidden_subflags[i].print() << endl;
    //}
}













void extra_arguments(int extra, int processed, int argc)
{
    if (processed+extra >= argc)
    {
        cerr << "Expected more arguments..." << endl;
        exit(1);
    }
}




int main(int argc, char *argv[])
{
    // Parse command line
    int Kn = 0;
    string objective_file = "";
    string output_file = "";
    bool verbose_output = false;
    bool use_sdp_temp = false;
    bool use_sdpa = false;
    
    bool dump_unlabeled = false;
    bool dump_unlabeled_while_generating = false;
    bool quit_after_generating_unlabeled = false;
    bool quit_after_generating_labeled = false;
    bool force_generate_flags = false;
    bool force_generate_labeled_flags = false;
    bool remove_duplicates_while_loading = false;
    bool types_from_file = false;
    
    bool upper_bound = false;
    bool lower_bound = false;
    bool force_generating_constriants = false;
    
    bool don_run_csdp = false;


    bool no_slack_flags = false;
    string no_slack_flags_filename = "";

    
    string csdp_binary="csdp";
    

    
    vector<string> additional_constraints;
    
    string csdp_OMP = "";
    

    if (argc <= 1)
    {
        cerr << endl;
        cerr << "Usage: " << argv[0] << " -n SIZE_OF_UNLABELED_FLAGS [-obj OBJECTIVE_FILENAME] [-v] [-o OUTPUT_DAT-S] [-lb | -ub]" << endl;
        cerr << " -n SIZE_OF_UNLABELED_FLAGS   Size of unlabeled graphs" << endl;
        cerr << " -obj OBJECTIVE_FILENAME      File containing objective function" << endl;
        cerr << " -v                           Display progress when creating SDP and generating graphs" << endl;
        cerr << " -o OUTPUT_DAT-S              Write sdp to a file OUTPUT_DAT-S, usable 'stdout', default test.dat-s" << endl;
        cerr << " -ub                          Creates SDP computing UPPER BOUND on the objective" << endl;        
        cerr << " -lb                          Creates SDP computing LOWER BOUND on the objective" << endl;
        cerr << " -fp                          Stores flag products or tries to reuse them - usefull for repeated computations, eats more space" << endl;
        cerr << " -csdp CSDPBIN                Command to run when running csdp. Default is csdp." << endl;
        cerr << " -csdp_OMP N                  Restricts the number of threads in csdp by setting\n"
                "                              OMP_NUM_THREADS=N before executing OMP." << endl;
        cerr << endl;
        
        
        cerr << "Filenames:" << endl;
        cerr << "    " << filename_prefix() << "__n?_unlabeled.txt    unlabeled flags" << endl;
        cerr << "    " << filename_prefix() << "__n?_labeled.txt      labeled flags" << endl;
        cerr << "    " << filename_prefix() << "__n?_sdp_products.txt part of sdp" << endl;
        cerr << "    " << filename_prefix() << "__forbidden.txt       forbidden flags" << endl;
        cerr << "    " << filename_prefix() << "__objective.txt       default objective" << endl;
        cerr << endl;
        return 0;
    }  
    
    
    int i = 1;
    while (i < argc)
    {        
        if (string(argv[i]) == "-n") {
            extra_arguments(1,i,argc);
            Kn = atoi(argv[i+1]);
            i++;
        } else if (string(argv[i]) == "-obj") {
            extra_arguments(1,i,argc);
            objective_file = argv[i + 1];
            i++;            
        } else if (string(argv[i]) == "-o") {
            extra_arguments(1,i,argc);
            output_file = argv[i + 1];
            i++;
        } else if (string(argv[i]) == "-csdp") {
            extra_arguments(1,i,argc);
            csdp_binary = argv[i + 1];
            i++;
        } else if (string(argv[i]) == "-v") {
            verbose_output = true;
        } else if (string(argv[i]) == "-lb") {
            lower_bound = true;
        } else if (string(argv[i]) == "-ub") {
            upper_bound = true;
        } else if (string(argv[i]) == "-rd") {
            remove_duplicates_while_loading = true;
        } else if (string(argv[i]) == "-du") {
            dump_unlabeled = true;
        } else if (string(argv[i]) == "-dug") {
            dump_unlabeled_while_generating = true;
        } else if (string(argv[i]) == "-qagu") {
            quit_after_generating_unlabeled = true;
        } else if (string(argv[i]) == "-qagl") {
            quit_after_generating_labeled = true;          
        } else if (string(argv[i]) == "-ncsdp") {
            don_run_csdp = true; 
        } else if (string(argv[i]) == "-nocsdp") {
            don_run_csdp = true; 
        } else if (string(argv[i]) == "-csdp_OMP") {
            extra_arguments(1,i,argc);
            csdp_OMP = argv[i + 1];
            i++;
        } else if (string(argv[i]) == "-sdpa") {
            use_sdpa = true;     
        } else if (string(argv[i]) == "-fgf") {
            force_generate_flags = true;
            force_generate_labeled_flags = true;
        } else if (string(argv[i]) == "-tff") {
            types_from_file = true;
        } else if (string(argv[i]) == "-fglf") {
            force_generate_labeled_flags = true;
        } else if (string(argv[i]) == "-fgc") {
            force_generating_constriants = true;
        } else if (string(argv[i]) == "-fp") {
            use_sdp_temp = true;
        } else {
            cerr << "Unsupported argument " << argv[i] << " " << endl;
            return 0;
        }
        i++;
    }
    
    if (i > argc)
    {
        cerr << "Probably some missing argument." << endl;
        return 1;
    }

    
    
    
    
    load_forbidden();

    
   
    
    //////////////////////////////// Normal use....
    
    
    if (objective_file == "")
    {
        stringstream filename;
        filename <<  filename_prefix() << "__objective.txt";
    
        objective_file = filename.str();
        cerr << "Objective file not provided, using default filename " << objective_file << endl;
    }
    
    if (!load_objective_from_file(objective_file, verbose_output))
    {
        return 1;
    }    
    
//#ifdef USE_FOR_CROSSINGS
//    cerr << "Warning: using special function for crossings - not everything works." << endl;
//    return main_crossings(Kn);
//#endif
    
    assert(Kn <= V);
        
   

    get_unlabeled_flags_of_size(Kn, force_generate_flags, remove_duplicates_while_loading, verbose_output, dump_unlabeled_while_generating);
    if (dump_unlabeled) dump_unlabeled_flags(Kn);
    if (quit_after_generating_unlabeled)
    {
        cerr << "Quitting after generating unlabeled flags." << endl;
        return 0;
    }

    
    
    
    ///////// Now the standard thing......
    
    // labeled flags
    g_flags.reserve(MAX_FLAG_TYPES); // Should make live much faster
    if (force_generate_labeled_flags || !load_labeled_flags_from_file(Kn,true))
    {

        cerr << "Generating labeled flags..." << endl;
        // Generating all labeled flags:
        int last_types = (int)g_flags.size();
//// TODO: Make this parallel        
        for (int i = 1; i <= Kn/2; i++)
        {
#ifdef DISABLE_UNLABELED_PRODUCTS
            if (Kn-2*i == 0) continue;
#endif
            cerr << "Getting labeled flags of size " << Kn-i << ":" << Kn-2*i << endl;
            generate_labeled_flags(Kn-i,Kn-2*i, verbose_output, types_from_file);
            int gain = (int)g_flags.size() - last_types;
            cerr << "Got "  << gain << " types ";
            for (int j = last_types; j < (int)g_flags.size(); j++)
                cerr << g_flags[j].size() << " ";
            cerr << endl;
            last_types = (int)g_flags.size();
        }
//#endif
        dump_labeled_flags(Kn);
    }
    cerr << "Labeled flag have " << (int)g_flags.size() << " types. Counts: ";
    for (int i = 0; i < (int)g_flags.size(); i++) cerr << " " << g_flags[i].size();
    cerr << endl;
    if (g_flags.size() == 0)
    {
        cerr << "WARNING: No labeled flags. Computations might not work." << endl;
    }
    if (quit_after_generating_labeled)
    {
        return 0;
    }



#ifdef WRITE_INTEGER_DENSITIES_FOR_LINEAR_CONSTRAINTS
    scale_linear_constraints_to_integers(Kn);
#endif



    
    
    if (!upper_bound && !lower_bound)
    {
        cerr << "I dont know if I should try upper bound our lower bound - you need to tell me using -lb or -ub" << endl;
        return 1;
    }
    
    if (upper_bound && lower_bound)
    {
        cerr << "I'm sorry, but I cannot do both upper and lower bound at the same time." << endl;
        return 1;
    }    
    
    
    // 
    if (no_slack_flags)
    {
        load_flags_from_file(no_slack_flags_filename, g_no_slack_flags);
        cerr << "Loaded list of " << g_no_slack_flags.size() << " flags with no slack" << endl;
    }
    
    // create result_file
    if (output_file == "" )
    {
        stringstream filename;
        filename << "SDP_n" << Kn << "_";
        if (lower_bound) filename << "LB_";
        else  filename << "UB_";
        filename <<  objective_file << ".dat-s"; 
        output_file = filename.str();
    }
    stringstream filename;
    filename << output_file << ".result";
    string result_file = filename.str();   
    
    
    
        
    if (output_file == "stdout")
    {
        cerr << "Generating SDP program to stdout..." << endl;
        print_CSDP(Kn,upper_bound,cout,verbose_output,use_sdp_temp);

#ifdef USE_COUNT_INSTEAD_OF_DENSITY_IN_OBJECTIVE
        cerr << "Warning - using count instead of density in SDP" << endl;
#endif
        
    } else {
        if (output_file == "" )
        {
            stringstream filename;
            filename << "SDP_n" << Kn << "_";
            if (lower_bound) filename << "LB_";
            else  filename << "UB_";
            filename <<  objective_file << ".dat-s"; 
            output_file = filename.str();
        }
        stringstream filename;
        filename << output_file << ".result";
        string result_file = filename.str();
               
        ofstream outfile;
        outfile.open (output_file.c_str(), ofstream::out);
        if (!outfile.good())
        {
            cerr << "Failed opening file " << output_file << endl;
            return -1;
        }
        cerr << "Generating SDP program to " << output_file << endl;
        print_CSDP(Kn,upper_bound,outfile,verbose_output,use_sdp_temp);
        outfile.close();
        
#ifdef USE_COUNT_INSTEAD_OF_DENSITY_IN_OBJECTIVE
        cerr << "Warning - using count instead of density in SDP" << endl;
#endif
        
        if (!don_run_csdp)
        {
            if (!csdp_OMP.empty())
            {
                setenv("OMP_NUM_THREADS", csdp_OMP.c_str(), 1);
            }
            
            
            if (use_sdpa)
            {
                execlp("sdpa","sdpa","-ds",output_file.c_str(),"-o",result_file.c_str(),(char *)NULL);                
            }
            else
            {
                execlp(csdp_binary.c_str(),"csdp",output_file.c_str(),result_file.c_str(),(char *)NULL);
            }
        }
        //execlp("sdpa","sdpa","-ds",output_file.c_str(),"-o",result_file.c_str(),(char *)NULL);
        //system("csdp",output_file.c_str());
    }
    
    cerr << "Done." << endl;
    
    
    return 0;    
}




